/**
 * Master Login
 *
 * @module package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/windows/Confirm',

    'package/quiqqer/masterlogin/bin/js/frontend/Manager',
    'package/quiqqer/masterlogin/bin/js/frontend/controls/UserSelectPopup',

    'Locale',
    'Ajax',
    'Mustache',

    'text!package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin.html',
    'css!package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin.css'

], function (QUI, QUIControl, QUIConfirm, MasterLoginManager, UserSelectPopup, QUILocale, QUIAjax, Mustache, template) {
    "use strict";

    var lg = 'quiqqer/masterlogin';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin',

        Binds: [
            '$onInject',
            '$toggleVisibility',
            '$openUserSelect',
            '$openLogBackConfirm'
        ],

        options: {
            slave     : false,      // set to true if user is logged in as slave (shows logback option and warning)
            masteruser: '',         // Info Text about master user
            slaveuser : ''          // Info Text about slave user
        },

        initialize: function (options) {
            this.parent(options);

            this.$visible = true;
            this.$Content = null;

            this.addEvents({
                onInject: this.$onInject
            });
        },

        /**
         * Event: onInject
         */
        $onInject: function () {
            this.$Content = new Element('div', {
                'class': 'quiqqer-masterlogin',
                html   : Mustache.render(template, {
                    slave          : this.getAttribute('slave') ? true : false,
                    slaveInfo      : QUILocale.get(lg, 'controls.MasterLogin.slave_info', {
                        slaveUser: this.getAttribute('slaveuser')
                    }),
                    logInLabel     : QUILocale.get(lg, 'controls.MasterLogin.tpl.logInLabel'),
                    logBackLabel   : QUILocale.get(lg, 'controls.MasterLogin.tpl.logBackLabel'),
                    visibilityTitle: QUILocale.get(lg, 'controls.MasterLogin.tpl.visibilityTitle')
                })
            }).inject(document.body, 'bottom');

            // Register events
            this.$Content.getElement('.quiqqer-masterlogin-visibility').addEvent('click', this.$toggleVisibility);

            // Determine initial visibility
            var visible = localStorage.getItem('quiqqer-masterlogin-visibility');

            if (visible == 0) {
                this.$toggleVisibility();
            }

            // Register action clicks
            if (this.getAttribute('slave')) {
                this.$Content.getElement(
                    '.quiqqer-masterlogin-actions-logback'
                ).addEvent('click', this.$openLogBackConfirm);

                this.$Content.addClass('quiqqer-masterlogin__slave');
                this.$Content.getElement('.quiqqer-masterlogin-visibility').addClass('quiqqer-masterlogin-visibility__slave')
            } else {
                this.$Content.getElement(
                    '.quiqqer-masterlogin-actions-login'
                ).addEvent('click', this.$openUserSelect);
            }
        },

        /**
         * Toggle masterlogin bar visibility
         */
        $toggleVisibility: function () {
            var ActionsContainer = this.$Content.getElement('.quiqqer-masterlogin-actions'),
                InfoContainer    = this.$Content.getElement('.quiqqer-masterlogin-info'),
                VisibilityIcon   = this.$Content.getElement('.quiqqer-masterlogin-visibility > span');

            if (this.$visible) {
                ActionsContainer.setStyle('display', 'none');
                InfoContainer.setStyle('display', 'none');

                this.$Content.addClass('quiqqer-masterlogin__hidden');

                this.$visible = false;
                localStorage.setItem('quiqqer-masterlogin-visibility', '0');

                if (this.getAttribute('slave')) {
                    VisibilityIcon.removeClass('fa-eye');
                    VisibilityIcon.addClass('fa-exclamation-triangle quiqqer-masterlogin__slaveicon');
                }

                return;
            }

            ActionsContainer.setStyle('display', '');
            InfoContainer.setStyle('display', '');

            this.$Content.removeClass('quiqqer-masterlogin__hidden');
            localStorage.setItem('quiqqer-masterlogin-visibility', '1');

            this.$visible = true;

            if (this.getAttribute('slave')) {
                VisibilityIcon.addClass('fa-eye');
                VisibilityIcon.removeClass('fa-exclamation-triangle quiqqer-masterlogin__slaveicon');
            }
        },

        /**
         * Open popup to select slave user
         */
        $openUserSelect: function () {
            new UserSelectPopup({
                events: {
                    onUserSelect: function (userId, Popup) {
                        MasterLoginManager.logInAs(userId).then(function (success) {
                            window.location.reload();
                        });
                    }
                }
            }).open();
        },

        /**
         * Open popup to confirm log back
         */
        $openLogBackConfirm: function () {
            new QUIConfirm({
                maxHeight: 300,
                maxWidth : 600,
                autoclose: false,

                information: QUILocale.get(lg, 'controls.MasterLogin.logback.information', {
                    slaveuser : this.getAttribute('slaveuser'),
                    masteruser: this.getAttribute('masteruser'),
                }),
                title      : QUILocale.get(lg, 'controls.MasterLogin.logback.title'),
                texticon   : 'fa fa-sign-out',
                text       : QUILocale.get(lg, 'controls.MasterLogin.logback.text'),
                icon       : 'fa fa-sign-out',

                cancel_button: {
                    text     : QUILocale.get(lg, 'controls.UserSelectPopup.closeButtonText'),
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button    : {
                    text     : QUILocale.get(lg, 'controls.MasterLogin.logback.confirm'),
                    textimage: 'icon-ok fa fa-check'
                },
                events       : {
                    onSubmit: function (Win) {
                        Win.Loader.show();

                        MasterLoginManager.logBack().then(function () {
                            window.location.reload();
                        }, function () {
                            Win.Loader.hide();
                        });
                    }
                }

            }).open();
        }
    });
});