<?php

namespace QUI\MasterLogin;

use QUI;

/**
 * Class Events
 *
 * Main event handler for quiqqer/masterlogin
 */
class Events
{
    /**
     * quiqqer/quiqqer: onTemplateGetHeader
     *
     * Load MasterLogin control in template
     *
     * @param QUI\Template $Template
     * @return void
     */
    public static function onTemplateGetHeader(QUI\Template $Template)
    {
        if (!QUI::getUsers()->isAuth(QUI::getUserBySession()) || !Manager::canUse()) {
            return;
        }

        $masterUser = '';
        $slaveUser  = '';

        if (Manager::isLoggedInAsSlave()) {
            // Fetch master user info
            $masterUserId = QUI::getSession()->get(Manager::SESSION_MASTER_USER_ID);

            try {
                $MasterUser = QUI::getUsers()->get($masterUserId);
                $masterUser = $MasterUser->getName().' (#'.$MasterUser->getId().')';
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }

            // Fetch slave user info (session user)
            /** @var QUI\Users\User $SlaveUser */
            $SlaveUser    = QUI::getUserBySession();
            $slaveName    = $SlaveUser->getName();
            $slaveCompany = '';

            try {
                $SlaveUserAddress = $SlaveUser->getStandardAddress();
                $slaveCompany     = $SlaveUserAddress->getAttribute('company');
                $slaveName        = $SlaveUserAddress->getName();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeDebugException($Exception);
            }

            $slaveUser = $slaveName.' - '.$slaveCompany.' (#'.$SlaveUser->getId().')';
        }

        $Template->extendFooter(
            '<div data-qui="package/quiqqer/masterlogin/bin/js/frontend/controls/MasterLogin"
                  data-qui-options-slave="'.(Manager::isLoggedInAsSlave() ? 1 : 0).'"
                  data-qui-options-masteruser="'.$masterUser.'"
                  data-qui-options-slaveuser="'.$slaveUser.'"
             ></div>'
        );
    }

    /**
     * quiqqer/quiqqer: onAdminLoad
     *
     * If the user accesses the backend -> log out of any possible slave user
     *
     * @return void
     */
    public static function onAdminLoad()
    {
        Manager::logBack();
    }

    /**
     * quiqqer/quiqqer: onUserLogout
     *
     * @param QUI\Users\User $User
     * @return void
     */
    public static function onUserLogout(QUI\Users\User $User)
    {
        // Update log entry
        try {
            QUI::getDataBase()->update(
                Manager::getLogTable(),
                [
                    'end' => \date('Y-m-d H:i:s')
                ],
                [
                    'end'      => null,
                    'slave_id' => $User->getId(),
                    'sec_hash' => QUI::getUsers()->getSecHash()
                ]
            );
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }
}
